package client_server;

/**
 * The Simple Web Server opens a port and gets a new ServerSocket.  When a web page client
 * opens a socket on the same port, it accepts the connection and reads the url string.
 * It then gets a string tokenizer and looks at the first two tokens.  If the name of the 
 * processor is not favicon.ico, it creates a new server thread and starts it.  This is done
 * in an infinite loop, which can only be stopped by closing the console window.
**/
import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.event.*; 
import java.awt.*;

public class WebServer extends Frame
{
	private Panel panel;
	private TextField portBox;
	private TextArea area;
	private Button startServer;
	private int count = 1; // Track the number of clients.
	
	public WebServer ()
	{
		super ("Web Server"); 
		setSize (800, 400); 

		// Get the Text Area and add it to the panel. 
		area = new TextArea (10, 80);
		add (area, BorderLayout.CENTER);
		
		// Get panel for text box and button.
		panel = new Panel ();
		panel.setSize (10, 2);
		panel.setBackground (Color.cyan);
		
		// Get a label for the port.
		Label portLabel = new Label ("You can change the port by modifying the number 8080 in the box.");
		panel.add (portLabel);
		// Get the text box for the port.
		portBox = new TextField ("8080", 8);
		panel.add (portBox);
		portBox.setSize (8, 2);
		// Get a new button and add a listener to it. 
		startServer = new Button ("Start Server"); 
		startServer.setSize (10, 2);
		startServer.addActionListener (new StartListener ()); 
		panel.add (startServer); 
 		add (panel, BorderLayout.NORTH);
	} //constructor
	
	/* Inner class to listen for the startServer button.
	 * When the button is clicked, a server socket is opened.  It waits in a while 
	 * loop until a client attempts to connect with it.  When one does, it reads the
	 * url string and extracts the method and the name of the processor.  If the 
	 * name is a valid process name, a new server thread is started, which processes
	 * the request.
	 */
	class StartListener implements ActionListener 
	{ 
		public void actionPerformed (ActionEvent event) 
		{
			int port = Integer.parseInt (portBox.getText ());
			try
			{
				ServerSocket serverSocket = new ServerSocket (port);
				while (true)
				{
					Socket clientSocket = serverSocket.accept (); // Respond to the client.
					// Get the URL string and tokenize it.
					BufferedReader in = new BufferedReader (new InputStreamReader 
						(clientSocket.getInputStream ()));
						
					// Get the url string and a tokenizer for it.
					String urlString = in.readLine();
					StringTokenizer tokenizer = new StringTokenizer(urlString, "/?&= ");
					
					// Get the first two tokens.
					String method = tokenizer.nextToken();
					String processName = tokenizer.nextToken();
					
					// Use only process names other than favicon.ico.
					if (!processName.equals ("favicon.ico"))
					{
						// Display data about the process.
						area.append (urlString + "\n");
						area.append (method + "\n");	
						area.append (processName + "\n");
					
						// Start the next client.
						area.append ("Client " + count + " starting:\n");
						new Server (tokenizer, processName, clientSocket, area).start ();
						count ++;
					}
				}
			} catch (IOException e) {area.append ("IO Exception\n");}
			catch (NumberFormatException e) {area.append ("Number error\n");}
		} // method actionPerformed 
	} // class StartListener 
  
	// The main method gets a new frame and shows it. 
	public static void main (String [] args) 
	{ 
     	System.out.println ("To stop the server, close this console window.");
		WebServer frame = new WebServer (); 
		frame.setVisible (true); 
	} 
} // WebServer

/**
 * The Server class is a thread.  It uses the tokenizer parameter to parse the string.
 * The tokens are sent to the Request class for further processing.  The process method
 * in the processor class is then started.  When the processor is finished, the socket
 * is closed.
**/

class Server extends Thread
{
	private WebRequestProcessor processor;
	private String processName;
	private Socket clientSocket;
	private TextArea area;
	private StringTokenizer tokenizer;
	//private Request request;
	//private Response response;
	
	public Server (StringTokenizer t, String name, Socket socket, TextArea a)
	{
		tokenizer = t;	
		processName = name;
		clientSocket = socket;
		area = a;
	} // constructor
	
	public void run ()
	{
		try
		{
			// Set up the Request and Response clases.
			Request request = new Request (tokenizer, area);
			Response response = new Response (clientSocket.getOutputStream ());
			
			// Start processing.	
			area.append ("Processing: " + processName + "\n");
			processor = (WebRequestProcessor) Class.forName (processName).newInstance ();
			processor.process (request, response);
			clientSocket.close (); // Close the client's socket.
		} catch (IOException e) {System.out.println ("Processing Exception");}
		catch (ClassNotFoundException ce) {area.append ("Class not found\n");}
		catch (InstantiationException ie) {area.append ("Instantiation exception\n");}
		catch (IllegalAccessException ie) {area.append ("Illegal Access Exception\n");}
		area.append ("Client at "+ clientSocket.getInetAddress() + " disconnected\n");
	} // run
} // class Server



